@@ -172,28 +172,33 @@ class SFBulkType:
         else:
             yield result.json()
 
-    def worker(self, batch, operation, wait=5):
+    def worker(self, batch, operation, wait=5, bypass_results=False):
         """ Gets batches from concurrent worker threads.
         self._bulk_operation passes batch jobs.
         The worker function checks each batch job waiting for it complete
         and appends the results.
         """
-
-        batch_status = self._get_batch(job_id=batch['jobId'],
-                                       batch_id=batch['id'])['state']
-
-        while batch_status not in ['Completed', 'Failed', 'NotProcessed']:
-            sleep(wait)
+        if not bypass_results:
             batch_status = self._get_batch(job_id=batch['jobId'],
                                            batch_id=batch['id'])['state']
 
-        batch_results = self._get_batch_results(job_id=batch['jobId'],
-                                                batch_id=batch['id'],
-                                                operation=operation)
-        result = batch_results
+            while batch_status not in ['Completed', 'Failed', 'NotProcessed']:
+                sleep(wait)
+                batch_status = self._get_batch(job_id=batch['jobId'],
+                                               batch_id=batch['id'])['state']
+
+            batch_results = self._get_batch_results(job_id=batch['jobId'],
+                                                    batch_id=batch['id'],
+                                                    operation=operation)
+            result = batch_results
+        else:
+            result = [{
+                          'bypass_results': bypass_results,
+                          'job_id': batch['jobId']
+                          }]
         return result
 
-    def _add_autosized_batches(self, operation, data, job):
+    def _add_autosized_batches(self, data, operation, job):
         """
         Auto-create batches that respect bulk api V1 limits.
 
@@ -219,45 +224,35 @@ class SFBulkType:
         * Maximum number of characters in a record: 400,000
         * Maximum number of characters in a field: 131,072
         """
-        file_limit = 1024 * 1024 * 10 # 10MB in bytes
+        file_limit = 1024 * 1024 * 10  # 10MB in bytes
         rec_limit = 10000
         char_limit = 10000000
 
         batches = []
         last_break = 0
-        nrecs, outsize, outchars = 0, 0, 0
+        nrecs, recsize = 0, 0
         for i, rec in enumerate(data):
             # 2 is added to account for the enclosing `[]`
             # and the separator `, ` between records.
-            recsize = len(json.dumps(rec, default=str)) + 2
-            recchars = str(rec) + 2
+            recsize += len(json.dumps(rec, default=str)) + 2
+            nrecs += len(rec)
             if any([
-                outsize + recsize > file_limit,
-                outchars + recchars > char_limit,
-                nrecs > rec_limit
-            ]):
-                batches.append(
-                    self._add_batch(
-                        job_id=job['id'],
-                        data=data[last_break:i],
-                        operation=operation
-                    )
-                )
+                recsize >= file_limit,
+                recsize >= char_limit,
+                nrecs >= rec_limit,
+                i >= len(data) - 1
+                ]):
+                i = 1 if len(data) == 1 else i
+                batches.append(data[last_break:i])
                 last_break = i
-                nrecs, outsize, outchars = 0, 0, 0
-        if last_break < len(data):
-            batches.append(
-                self._add_batch(
-                    job_id=job['id'],
-                    data=data[last_break:len(data)],
-                    operation=operation
-                )
-            )
-        return batches
+                nrecs, recsize = 0, 0
+        return [self._add_batch(job_id=job, data=i,
+                                operation=operation) for i in batches]
 
     # pylint: disable=R0913
     def _bulk_operation(self, operation, data, use_serial=False,
-                        external_id_field=None, batch_size=10000, wait=5):
+                        external_id_field=None, batch_size=10000, wait=5,
+                        bypass_results=False):
         """ String together helper functions to create a complete
         end-to-end bulk API request
         Arguments:
@@ -275,6 +270,10 @@ class SFBulkType:
             raise ValueError('batch size should be auto or an integer')
 
         if operation not in ('query', 'queryAll'):
+            # Checks if data is present
+            if not data:
+                raise ValueError(f'data should not be empty for {operation}')
+
             # Checks to prevent batch limit
             if batch_size != 'auto':
                 batch_size = min(batch_size, len(data), 10000)
@@ -285,7 +284,9 @@ class SFBulkType:
                                        use_serial=use_serial,
                                        external_id_field=external_id_field)
                 if batch_size == 'auto':
-                    batches = self._add_autosized_batches(operation, data, job)
+                    batches = self._add_autosized_batches(job=job['id'],
+                                                          data=data,
+                                                          operation=operation)
                 else:
                     batches = [
                         self._add_batch(job_id=job['id'], data=i,
@@ -296,11 +297,14 @@ class SFBulkType:
 
                 multi_thread_worker = partial(self.worker,
                                               operation=operation,
-                                              wait=wait)
+                                              wait=wait,
+                                              bypass_results=bypass_results)
                 list_of_results = pool.map(multi_thread_worker, batches)
 
                 results = [x for sublist in list_of_results for i in
-                           sublist for x in i]
+                           sublist for x in i] if not bypass_results else \
+                    [{k: v} for sublist in list_of_results for i in
+                     sublist for k, v in i.items()]
 
                 self._close_job(job_id=job['id'])
 
@@ -335,7 +339,8 @@ class SFBulkType:
         return results
 
     # _bulk_operation wrappers to expose supported Salesforce bulk operations
-    def delete(self, data, batch_size=10000, use_serial=False):
+    def delete(self, data, batch_size=10000, use_serial=False,
+               bypass_results=False):
         """ soft delete records
 
         Data is batched by 10,000 records by default. To pick a lower size
@@ -344,11 +349,12 @@ class SFBulkType:
         """
         results = self._bulk_operation(use_serial=use_serial,
                                        operation='delete', data=data,
-                                       batch_size=batch_size)
+                                       batch_size=batch_size,
+                                       bypass_results=bypass_results)
         return results
 
     def insert(self, data, batch_size=10000,
-               use_serial=False):
+               use_serial=False, bypass_results=False):
         """ insert records
 
         Data is batched by 10,000 records by default. To pick a lower size
@@ -357,11 +363,12 @@ class SFBulkType:
         """
         results = self._bulk_operation(use_serial=use_serial,
                                        operation='insert', data=data,
-                                       batch_size=batch_size)
+                                       batch_size=batch_size,
+                                       bypass_results=bypass_results)
         return results
 
     def upsert(self, data, external_id_field, batch_size=10000,
-               use_serial=False):
+               use_serial=False, bypass_results=False):
         """ upsert records based on a unique identifier
 
         Data is batched by 10,000 records by default. To pick a lower size
@@ -371,10 +378,12 @@ class SFBulkType:
         results = self._bulk_operation(use_serial=use_serial,
                                        operation='upsert',
                                        external_id_field=external_id_field,
-                                       data=data, batch_size=batch_size)
+                                       data=data, batch_size=batch_size,
+                                       bypass_results=bypass_results)
         return results
 
-    def update(self, data, batch_size=10000, use_serial=False):
+    def update(self, data, batch_size=10000, use_serial=False,
+               bypass_results=False):
         """ update records
 
         Data is batched by 10,000 records by default. To pick a lower size
@@ -383,10 +392,12 @@ class SFBulkType:
         """
         results = self._bulk_operation(use_serial=use_serial,
                                        operation='update', data=data,
-                                       batch_size=batch_size)
+                                       batch_size=batch_size,
+                                       bypass_results=bypass_results)
         return results
 
-    def hard_delete(self, data, batch_size=10000, use_serial=False):
+    def hard_delete(self, data, batch_size=10000, use_serial=False,
+                    bypass_results=False):
         """ hard delete records
 
         Data is batched by 10,000 records by default. To pick a lower size
@@ -395,21 +406,23 @@ class SFBulkType:
         """
         results = self._bulk_operation(use_serial=use_serial,
                                        operation='hardDelete', data=data,
-                                       batch_size=batch_size)
+                                       batch_size=batch_size,
+                                       bypass_results=bypass_results)
         return results
 
-    def query(self, data, lazy_operation=False):
+    def query(self, data, lazy_operation=False, wait=5):
         """ bulk query """
-        results = self._bulk_operation(operation='query', data=data)
+        results = self._bulk_operation(operation='query', data=data, wait=wait)
 
         if lazy_operation:
             return results
 
         return list_from_generator(results)
 
-    def query_all(self, data, lazy_operation=False):
+    def query_all(self, data, lazy_operation=False, wait=5):
         """ bulk queryAll """
-        results = self._bulk_operation(operation='queryAll', data=data)
+        results = self._bulk_operation(operation='queryAll', data=data,
+            wait=wait)
 
         if lazy_operation:
             return results
